#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

module RailsUtils
  module Model
    # A base class for models that are not mapped to database tables. Models
    # that extend this class may take advantage of active record validation
    # support, but will not be saved to the database.
    #     
    # An advantage of doing this is that your models can still be used by
    # controllers without changing the approach of the controller code.  This
    # is especially useful with libraries such as make_resourceful as it still
    # allows you to take advantage of all the dynamic code that
    # make_resourceful creates for you.
    #
    # If a tableless model wishes to simulate AR CRUD protocol/actions, it
    # should override one ore more of the following methods:
    #
    # * Class-level:
    #   * +find+: Handle all the find behavior yourself, or
    #   * +list+: Handle :all/:first, with built-in handling for find by id
    #     given that you implement the +load+ method.
    # * Instance-level:
    #   * +load+: new hook method used to allow an object to load itself. 
    #     Used by Tableless.find when given an id.
    #   * +create+: create the object, called when object is saved and
    #     new_record? returns true.
    #   * +update+: update the object, called when object is saved and
    #     new_record? returns false.
    #   * +destroy+: destroy the object
    class Tableless < ::ActiveRecord::Base
      module FinderTemplate
        def build(*args)
          new(*args)
        end

        def find(*args)
          case args.first
          when Symbol
            list(*args)
          when nil
            nil
          else
            objects = args.map do |id|
              next if id.nil?
              obj = build
              obj.id = id
              # hint that we're doing a load
              obj.new_record = false
              obj.load
              obj
            end.compact
            case objects.size
            when 0
              nil
            when 1
              objects.first
            else
              objects
            end
          end
        end

        def list(*args)
          raise "find #{args.first} not implemented"
        end
      end

      # HasManyLoader can be used as a custom association loader to simulate
      # <tt>has_many</tt> relationships in a tableless model.  Use as follows:
      #
      #     class User < RailsUtils::Model::Tableless
      #       has_many :memberships, :extend => ::Membership::AssociationLoader
      #     end
      #
      # where <tt>Membership::AssociationLoader looks like the following:
      #
      #     class Membership < RailsUtils::Model::Tableless
      #       module AssociationLoader
      #         include RailsUtils::Model::Tableless::HasManyLoader
      #         def build(*args)
      #           # create a membership here
      #         end
      #       end
      #     end
      module HasManyLoader 
        include FinderTemplate
        def construct_sql; end
      end

      # Here we override find plus a few more class methods that allow us to
      # disable database access and declare columns in code
      class << self
        include FinderTemplate

        def transaction
          # No database transactions
          yield
        end

        def columns
          @columns ||= []
        end

        def column(name, sql_type=nil, default=nil, null=true)
          columns << ActiveRecord::ConnectionAdapters::Column.new(
          name.to_s, default, sql_type.to_s, null)
        end
      end

      def load(*args)
        raise ActiveRecord::RecordNotFound, "load not implemented by #{self.class}"
      end

      attr_writer :new_record

      def create
        @new_record = false
        id
      end

      def update
      end

      def destroy
        freeze
      end
    end
  end
end
