#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

require 'net/https'

# RailsUtils::Net::ConnectionError is a generic base class for
# reporting connection errors
class RailsUtils::Net::ConnectionError < StandardError # :nodoc;

  def initialize(response, message = nil)
    @response = response
    @message = message
  end

  attr_reader :response

  def to_s
    "Failed with #{response.code}"
  end

end

# 4xx General Client Error
class RailsUtils::Net::ClientError < RailsUtils::Net::ConnectionError; end # :nodoc:

# 401 Unauthorized
class RailsUtils::Net::Unauthorized < RailsUtils::Net::ClientError; end # :nodoc:

# 404 Not Found
class RailsUtils::Net::ResourceNotFound < RailsUtils::Net::ClientError; end # :nodoc:

# 405 Method Not Allowed
class RailsUtils::Net::MethodNotAllowed < RailsUtils::Net::ClientError # :nodoc:
  def allowed_methods
    @response['Allow'].split(',').map { |verb| verb.strip.downcase.to_sym }
  end
end

# 409 Conflict
class RailsUtils::Net::ResourceConflict < RailsUtils::Net::ClientError; end # :nodoc:

# 422 Invalid Resource
class RailsUtils::Net::ResourceInvalid < RailsUtils::Net::ClientError; end # :nodoc:

# 5xx General Server Error
class RailsUtils::Net::ServerError < RailsUtils::Net::ConnectionError; end # :nodoc:

# RailsUtils::Net::Connection is an abstraction around the HTTP
# connection used to interact with a particular server.
#
# == Connection Creation:
#
# A new instance must be passed the beginning of a URI for the
# remote server to be contacted by the operation methods.  The
# +site+ value can be either a String or a URI.  This value can
# also be adjusted later, via the +site=+ method.
#
#   conn = RailsUtils::Net::Connection.new("http://localhost:3000")
#
# == Instance Attributes:
#
# All instance attributes listed here are read/write unless otherwise noted.
#
# +default_headers+::          Hash of name/value pairs represented HTTP
#                              headers that will be merged into every request.
#                              By default, this attribute sends
#                              "Content-Type: application/xml" on each request.
#
# +response+::                 The HTTP response from the last HTTP request
#                              that was performed via this Connection.  This
#                              attribute always reflects the response from the
#                              most recently completed operation method.
#
# +site+::                     Absolute URI prefix for operation method calls.
#
# == Operation Methods:
#
# The primary methods used to interact with the remote server are
# the following:
#
#   conn.get("/customers.xml")            Perform an HTTP GET
#   conn.post("/customers.xml", body)     Perform an HTTP POST
#   conn.put("/customers/1.xml", body)    Perform an HTTP PUT
#   conn.delete("/customers/1.xml")       Perform an HTTP DELETE
#
# Methods that return content (+get+) return the unmodified body.
# It is up to the caller to know what the content type is, and
# interpret the data appropriately.  In a similar manner, methods
# that send content (+post+, +put+) take an uninterpreted +body+
# parameter that is sent unchanged.
#
class RailsUtils::Net::Connection

  # ---------------------------------------------------------------- Constructor

  # The +site+ parameter is required, and will set the +site+ attribute
  # to the URI of the RESTful service we will communicate with
  def initialize(site)
    raise ArgumentError, 'Missing site URI' unless site
    @default_headers = { 'Content-Type' => 'application/xml' }
    @site = site.is_a?(URI) ? site : URI.parse(site)
  end

  # ------------------------------------------------ Public Instance Methods

  # Retrieve or set a hash of the default HTTP headers to be included
  # in every request.  If the operational requests also specify headers,
  # these will be merged (with the specified headers overriding the default
  # headers in cases of conflict).
  attr_accessor :default_headers

  # Perform an HTTP DELETE request, used to destroy resources.
  def delete(path, headers = {})
    request(:delete, path, headers(headers))
  end

  # Perform an HTTP GET request, used to retrieve (find) resources,
  # and return the response body.
  def get(path, headers = {})
    request(:get, path, headers(headers))
    @response.body
  end

  # Perform an HTTP POST request, used to create new resources.
  def post(path, body = '', headers = {})
    request(:post, path, body.to_s, headers(headers))
  end

  # Perform an HTTP PUT request, used to update existing resources.
  def put(path, body = '', headers = {})
    request(:put, path, body.to_s, headers(headers))
  end

  # Return the response returned by the most recent HTTP request
  # performed via this Connection.
  attr_reader :response

  attr_reader :site

  # Set the URI for the RESTful service we will communicate with
  def site=(site)
    puts "site is " + site.inspect
    @site = site.is_a?(URI) ? site : URI.parse(site)
    @http = nil
  end

  # Turn on debug, writing to the given IO device (stdout by default).
  # Debug for the duration of block execution if passed a block.
  def debug(out = $stdout)
    if block_given?
      begin
        http.set_debug_output out
        return_value = yield
      ensure
        http.set_debug_output nil
      end
      return_value
    else
      http.set_debug_output out
    end
  end

  # --------------------------------------------------- Private Instance Methods

  private

  # Return a hash containing the HTTP BASIC authentication header
  # to use (if the site URI includes credentials), else an empty hash.
  def authorization()
    (@site.user || @site.password ? { 'Authorization' => 'Basic' + ["#{@site.user}:#{ @site.password}"].pack('m').delete("\r\n") } : { } )
  end

  # Return a hash of HTTP headers to be sent, merging those in
  # +default_headers+ with those in +headers+ (with the latter taking
  # precedence in the case of conflict.
  def headers(headers = {})
    authorization.merge(default_headers).merge(headers)
  end

  # Create (if necessary) and return a Net::HTTP instance for
  # communication with remote services and resources.
  def http
    unless @http
      @http = Net::HTTP.new(@site.host, @site.port)
      @http.use_ssl = @site.is_a?(URI::HTTPS)
      @http.verify_mode = OpenSSL::SSL::VERIFY_NONE if @http.use_ssl
    end
    @http
  end

  # Peform an HTTP request to our remote service, and return the
  # corresponding response (or raise an exception if an error occurred).
  #
  # === Options:
  #
  # +method+::        The HTTP method to perform (+:get+, +:post+,
  #                   +:put+, or +:delete+).
  # +path+::          Path portion of the URI used to interact with
  #                   the server, starting with a slash.  This value
  #                   is resolved relative to the base URI in +site+.
  # +arguments+::     Additional arguments specific to the HTTP method
  #                   being used.  Most common arguments are the +body+
  #                   content to send (+:post+ or +:put+), and a Hash
  #                   of request headers to be included.
  #
  # Make an HTTP request to our remote service, and handle the
  # resulting response.
  def request(method, path, *arguments)
    @response = http.send(method, full_path(path), *arguments)
    case response.code.to_i
    when 200...400
      @response
    when 401
      raise RailsUtils::Net::Unauthorized.new(@response)
    when 404
      raise RailsUtils::Net::ResourceNotFound.new(@response)
    when 405
      raise RailsUtils::Net::MethodNotAllowed.new(@response)
    when 409
      raise RailsUtils::Net::ResourceConflict.new(@response)
    when 422
      raise RailsUtils::Net::ResourceInvalid.new(@response)
    when 400...500
      raise RailsUtils::Net::ClientError.new(@response)
    when 500...600
      raise RailsUtils::Net::ServerError.new(@response)
    else
      raise RailsUtils::Net::ConnectionError.new(@response, "Unknown response code: #{@response.code}")
    end
  end

  def full_path(partial)
    if @site.path
      "#{@site.path.sub(%r{/$},'')}#{partial}"
    else
      partial
    end
  end
end
