#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

require File.expand_path(File.dirname(__FILE__) + "/../../spec_helper")

def current_model_name
  "NamedThing"
end

def model_includes
  {}
end

require 'spec/rails'

describe RailsUtils::Controller::FilteredIndexHelper, "#page_params", :behaviour_type => :helper do
  it "should return a hash containing finder options for :limit and :offset" do
    params[:page] = "2"
    params[:size] = "15"
    hash = page_params
    hash[:limit].should == 15
    hash[:offset].should == 16

    params[:page] = "7"
    params[:size] = "11"
    hash = page_params
    hash[:limit].should == 11
    hash[:offset].should == 67
  end

  it "should default to DEFAULT_PAGE and DEFAULT_SIZE for the page and size" do
    hash = page_params
    hash[:limit].should == RailsUtils::Controller::FilteredIndexHelper::DEFAULT_SIZE
    hash[:offset].should == RailsUtils::Controller::FilteredIndexHelper::DEFAULT_PAGE
  end

  it "should allow use of limit and offset params directly to override page and size" do
    params[:page] = "2"
    params[:size] = "15"
    params[:limit] = "10"
    params[:offset] = "15"
    hash = page_params
    hash[:limit].should == 10
    hash[:offset].should == 15
  end
end

describe RailsUtils::Controller::FilteredIndexHelper, "#filter_params", :behaviour_type => :helper do
  it "should return params with standard keys removed" do
    standards = {:limit => "1", :offset => "10", :page => "1", :size => "10",
      :controller => "controller", :action => "action", :format => "format"}
    params.update standards
    filter_params[:conditions].should_not include(*standards.keys)
  end
end

describe RailsUtils::Controller::FilteredIndexHelper, "#include_params", :behaviour_type => :helper do
  it "should return an array of the included fields" do
    params.update :include => "account,children"
    include_params.should == { :include => [:account, :children] }
  end

  it "should filter out non-associations" do
    params.update :include => "account,name,bogus"
    include_params.should == { :include => [:account] }
  end

  it "should return an empty hash if no include parameter is present" do
    include_params.should == {}
  end
end

describe RailsUtils::Controller::FilteredIndexHelper, "#order_params", :behaviour_type => :helper do
  it "should return an ordering specification" do
    ordering = { :order => "name asc" }
    params.update ordering
    order_params.should == ordering
  end

  it "should filter out fields that don't exist" do
    params.update :order => "name, bogus desc"
    order_params.should == { :order => "name" }
  end

  it "should leave out any direction that doesn't match asc or desc" do
    params.update :order => "name bogus"
    order_params.should == { :order => "name" }
  end
end

describe RailsUtils::Controller::FilteredIndexHelper, "#options_for_find", :behaviour_type => :helper do
  before :each do
    params[:name] = "sample"
    params[:page] = 2
  end

  it "should return a hash containing conditions, limit, offset, include, and order" do
    params.update :include => "account", :order => "name"
    expected = {:limit => 10, :offset => 11, 
      :conditions => {"name" => "sample"},
      :include => [ :account ], :order => "name" }
    options_for_find.should == expected
  end

  it "should filter out any conditions that do not correspond to valid columns" do
    params[:parent_id] = "123"
    params[:bogus_param] = "bad"
    expected = {:limit => 10, :offset => 11,
      :conditions => {"name" => "sample", "parent_id" => "123"}}
    options_for_find.should == expected
  end
end

describe RailsUtils::Controller::FilteredIndexHelper, "#current_objects", :behaviour_type => :helper do
  before :each do
    @model = mock("model")
  end
  def model_includes; {}; end
  def current_model; @model; end
  def current_model_name; "NamedThing"; end
  
  it "should call find on the current model with contents of options_for_find" do
    options = options_for_find
    @model.should_receive(:find).with(:all, options).and_return []
    current_objects.should == []
  end
end

class NamedThing < RailsUtils::Model::Tableless
  column :name, :string
  column :parent_id, :string
  has_one :account ,:class_name => "NamedThing"
  has_many :children, :class_name => "NamedThing", :foreign_key => "parent_id"
end
