#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

require File.expand_path(File.dirname(__FILE__) + "/../../spec_helper")

describe RailsUtils::Model::Tableless do
  before :each do
    @tableless = Class.new(RailsUtils::Model::Tableless)
    @tableless.column :name, :string
    @tableless.column :description, :string
  end

  it "should raise an error on find :all if list is not implemented as a singleton method on the subclass" do
    lambda { @tableless.find :all }.should raise_error
  end

  it "should raise an error on find by id if load is not implemented in the subclass" do
    lambda { @tableless.find "id" }.should raise_error(ActiveRecord::RecordNotFound)
  end

  it "should instantiate a new object, set its id, and call load for find by id" do
    @tableless.module_eval do
      attr_reader :loaded
      def load
        @loaded = self.id
      end
    end
    obj = @tableless.find "id"
    obj.should be_a_kind_of(@tableless)
    obj.loaded.should == "id"
  end

  it "should return an array of objects if more than one id is given to find" do
    @tableless.module_eval do
      def load
      end
    end

    @tableless.find("1", "2").size.should == 2
  end

  it "should return nil if no ids are specified" do
    @tableless.find.should be_nil
  end

  it "should return an array of column names for #column_names" do
    @tableless.column_names.should include("name", "description")
  end

  it "should call create when saved as a new record" do
    @tableless.module_eval do
      attr_reader :created
      def create
        @created = true
        super
      end
    end
    obj = @tableless.new
    obj.should be_new_record
    obj.save
    obj.created.should be_true
    obj.should_not be_new_record
  end

  it "should call update when saved as an existing record" do
    @tableless.module_eval do
      attr_reader :updated
      def update
        @updated = true
        super
      end
    end
    obj = @tableless.new
    obj.send :instance_variable_set, "@new_record", false
    obj.should_not be_new_record
    obj.save
    obj.updated.should be_true
  end

  it "should return nil if find received nil" do
    @tableless.find(nil).should be_nil
  end

  it "should return an array with nils removed if nils are included" do
    @tableless.module_eval do
      def load
      end
    end

    @tableless.find("1", nil, "2", nil).size.should == 2
  end
end
