#--
# Copyright (C) 2007  Sun Microsystems, Inc.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#++

# Include this module in your controller to provide a custom #current_objects
# method that processes parameters to be passed to <tt>Model.find(:all).</tt>
#
# NOTE: it must currently be included <em>after</em> the <tt>make_resourceful</tt>
# declaration to take effect (thus, unfortunately you can't put it in your
# ApplicationController).
#
# Parameters used:
#
# * +page+: current page of data to return (integer).
# * +size+: size of page of data to return (integer).
# * +limit+: same as +size+
# * +offset+: actual row offset (zero-based, integer).  Equivalent to
#   <tt>(page - 1) * size + 1</tt>.
# * +include+: comma-separated list of one or more associations to include in
#   the fetch.
# * +order+: one or more <tt>field_name asc</tt> or <tt>field_name desc</tt>
#   phrases, comma-separated.  If the names given are not valid fields or the
#   phrase is not <tt>/(\w+)\s+(asc|desc)/, the phrase is dropped from the
#   ordering.
# * Any other parameters present, if they match the name of an attribute in
#   the model, are passed in a :conditions has to the find method, allowing
#   filtering by value. At this time nothing other than value equality is
#   supported.
module RailsUtils::Controller::FilteredIndexHelper
  DEFAULT_PAGE = 1
  DEFAULT_SIZE = 10
  REMOVED_PARAMS = %w(page size limit offset controller action format include order)

  def current_objects
    @current_objects ||= current_model.find(:all, options_for_find)
  end

  def current_model_columns
    @current_columns ||= current_model_name.constantize.columns
  end

  def current_model_associations
    @current_associations ||= current_model_name.constantize.reflect_on_all_associations
  end

  def filter_params
    p = {}
    params.each do |k,v|
      next if REMOVED_PARAMS.include? k
      next unless current_model_columns.detect {|c| c.name == k.to_s }
      p[k.to_sym] = v
    end
    { :conditions => p }
  end

  def page_params
    p = {}
    page, size = (params[:page] || DEFAULT_PAGE).to_i, (params[:size] || DEFAULT_SIZE).to_i
    p[:limit]  = size
    p[:offset] = (page - 1) * size
    p[:limit]  = params[:limit].to_i if params[:limit]
    p[:offset] = params[:offset].to_i if params[:offset]
    p
  end

  def include_params
    if params[:include]
      includes = params[:include].split(/\s*,\s*/).map {|i| i.to_sym }
      { :include => includes.select do |i| 
          current_model_associations.detect {|assoc| assoc.name == i}
        end }
    else
      model_includes
    end
  end

  def order_params
    if order = params[:order]
      fields = order.split(/\s*,\s/).map {|f| f.split(/\s+/, 2)}
      fields.reject! do |f|
        !current_model_columns.detect {|c| c.name == f[0] }
      end
      { :order => fields.inject("") do |ord,f|
          ord << ", " unless ord.empty?
          ord << "#{f[0]}"
          ord << " #{f[1]}" if f[1] && f[1] =~ /(asc|desc)/i
          ord
        end }
    else
      {}
    end
  end

  def options_for_find
    options = filter_params.merge(include_params).merge(page_params).merge(order_params)
    options.delete(:conditions) if options[:conditions].empty?
    options.delete(:include) if options[:include] && options[:include].empty?
    p options
    options
  end
end
